@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,6 @@
 #include "ibusimpl.h"
 #include "marshalers.h"
 #include "matchrule.h"
-#include "registry.h"
 #include "types.h"
 
 enum {
@@ -733,7 +732,8 @@ bus_dbus_impl_name_has_owner (BusDBusImpl           *dbus,
         g_dbus_method_invocation_return_error (invocation,
                                                G_DBUS_ERROR,
                                                G_DBUS_ERROR_FAILED,
-                                               "'%s' is not a legal bus name");
+                                               "'%s' is not a legal bus name",
+                                               name ? name : "(null)");
         return;
     }
 
@@ -1093,6 +1093,7 @@ bus_dbus_impl_release_name (BusDBusImpl           *dbus,
                             GDBusMethodInvocation *invocation)
 {
     const gchar *name= NULL;
+    BusNameService *service;
     g_variant_get (parameters, "(&s)", &name);
 
     if (name == NULL ||
@@ -1113,11 +1114,25 @@ bus_dbus_impl_release_name (BusDBusImpl           *dbus,
     }
 
     guint retval;
-    if (g_hash_table_lookup (dbus->names, name) == NULL) {
+    service = g_hash_table_lookup (dbus->names, name);
+    if (service == NULL) {
         retval = 2; /* DBUS_RELEASE_NAME_REPLY_NON_EXISTENT */
     }
     else {
+        /* "ReleaseName" method removes the name in connection->names
+         * and the connection owner.
+         * bus_dbus_impl_connection_destroy_cb() removes all
+         * connection->names and the connection owners.
+         * See also comments in bus_dbus_impl_connection_destroy_cb().
+         */
         if (bus_connection_remove_name (connection, name)) {
+            BusConnectionOwner *owner =
+                    bus_name_service_find_owner (service, connection);
+            bus_name_service_remove_owner (service, owner, dbus);
+            if (service->owners == NULL) {
+                g_hash_table_remove (dbus->names, service->name);
+            }
+            bus_connection_owner_free (owner);
             retval = 1; /* DBUS_RELEASE_NAME_REPLY_RELEASED */
         }
         else {
@@ -1188,9 +1203,8 @@ bus_dbus_impl_start_service_by_name (BusDBusImpl           *dbus,
         return;
     }
 
-    BusRegistry *registry = BUS_DEFAULT_REGISTRY;
-    BusComponent *component = bus_registry_lookup_component_by_name (registry,
-                                                                     name);
+    BusComponent *component = bus_ibus_impl_lookup_component_by_name (
+            BUS_DEFAULT_IBUS, name);
 
     if (component == NULL || !bus_component_start (component, g_verbose)) {
         g_dbus_method_invocation_return_error (invocation,
