@@ -25,9 +25,9 @@
 
 #include "engineproxy.h"
 #include "factoryproxy.h"
+#include "global.h"
 #include "ibusimpl.h"
 #include "marshalers.h"
-#include "option.h"
 #include "types.h"
 
 struct _SetEngineByDescData {
@@ -53,7 +53,6 @@ struct _BusInputContext {
     gchar *client;
 
     gboolean has_focus;
-    gboolean enabled;
 
     /* client capabilities */
     guint capabilities;
@@ -96,6 +95,7 @@ struct _BusInputContextClass {
     IBusServiceClass parent;
 
     /* class members */
+    IBusEngineDesc *default_engine_desc;
 };
 
 enum {
@@ -118,8 +118,6 @@ enum {
     CURSOR_DOWN_LOOKUP_TABLE,
     REGISTER_PROPERTIES,
     UPDATE_PROPERTY,
-    ENABLED,
-    DISABLED,
     ENGINE_CHANGED,
     REQUEST_ENGINE,
     LAST_SIGNAL,
@@ -144,31 +142,6 @@ static void     bus_input_context_service_method_call
                                                  const gchar            *method_name,
                                                  GVariant               *parameters,
                                                  GDBusMethodInvocation  *invocation);
-/*
-static GVariant *bus_input_context_service_get_property
-                                                (IBusService            *service,
-                                                 GDBusConnection        *connection,
-                                                 const gchar            *sender,
-                                                 const gchar            *object_path,
-                                                 const gchar            *interface_name,
-                                                 const gchar            *property_name,
-                                                 GError                **error);
-static gboolean bus_input_context_service_set_property
-                                                (IBusService            *service,
-                                                 GDBusConnection        *connection,
-                                                 const gchar            *sender,
-                                                 const gchar            *object_path,
-                                                 const gchar            *interface_name,
-                                                 const gchar            *property_name,
-                                                 GVariant               *value,
-                                                 GError                **error);
-*/
-static gboolean bus_input_context_filter_keyboard_shortcuts
-                                                (BusInputContext        *context,
-                                                 guint                   keyval,
-                                                 guint                   keycode,
-                                                 guint                   modifiers);
-
 static void     bus_input_context_unset_engine  (BusInputContext        *context);
 static void     bus_input_context_commit_text   (BusInputContext        *context,
                                                  IBusText               *text);
@@ -247,11 +220,6 @@ static const gchar introspection_xml[] =
     "    <method name='FocusIn' />"
     "    <method name='FocusOut' />"
     "    <method name='Reset' />"
-    "    <method name='Enable' />"
-    "    <method name='Disable' />"
-    "    <method name='IsEnabled'>"
-    "      <arg direction='out' type='b' name='enable' />"
-    "    </method>"
     "    <method name='SetCapabilities'>"
     "      <arg direction='in' type='u' name='caps' />"
     "    </method>"
@@ -275,8 +243,6 @@ static const gchar introspection_xml[] =
     "    <signal name='CommitText'>"
     "      <arg type='v' name='text' />"
     "    </signal>"
-    "    <signal name='Enabled'/>"
-    "    <signal name='Disabled'/>"
     "    <signal name='ForwardKeyEvent'>"
     "      <arg type='u' name='keyval' />"
     "      <arg type='u' name='keycode' />"
@@ -337,6 +303,16 @@ bus_input_context_class_init (BusInputContextClass *class)
 {
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (class);
 
+    class->default_engine_desc = ibus_engine_desc_new ("dummy",
+                                                       "",
+                                                       "",
+                                                       "",
+                                                       "",
+                                                       "",
+                                                       "ibus-engine",
+                                                       "");
+    g_object_ref_sink (class->default_engine_desc);
+
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_input_context_destroy;
 
     /* override the parent class's implementation. */
@@ -543,26 +519,6 @@ bus_input_context_class_init (BusInputContextClass *class)
             1,
             IBUS_TYPE_PROPERTY);
 
-    context_signals[ENABLED] =
-        g_signal_new (I_("enabled"),
-            G_TYPE_FROM_CLASS (class),
-            G_SIGNAL_RUN_LAST,
-            0,
-            NULL, NULL,
-            bus_marshal_VOID__VOID,
-            G_TYPE_NONE,
-            0);
-
-    context_signals[DISABLED] =
-        g_signal_new (I_("disabled"),
-            G_TYPE_FROM_CLASS (class),
-            G_SIGNAL_RUN_LAST,
-            0,
-            NULL, NULL,
-            bus_marshal_VOID__VOID,
-            G_TYPE_NONE,
-            0);
-
     context_signals[ENGINE_CHANGED] =
         g_signal_new (I_("engine-changed"),
             G_TYPE_FROM_CLASS (class),
@@ -597,7 +553,7 @@ bus_input_context_class_init (BusInputContextClass *class)
 static void
 bus_input_context_init (BusInputContext *context)
 {
-    context->prev_keyval = IBUS_VoidSymbol;
+    context->prev_keyval = IBUS_KEY_VoidSymbol;
     g_object_ref_sink (text_empty);
     context->preedit_text = text_empty;
     context->preedit_mode = IBUS_ENGINE_PREEDIT_CLEAR;
@@ -716,7 +672,7 @@ _ic_process_key_event  (BusInputContext       *context,
                         GVariant              *parameters,
                         GDBusMethodInvocation *invocation)
 {
-    guint keyval = IBUS_VoidSymbol;
+    guint keyval = IBUS_KEY_VoidSymbol;
     guint keycode = 0;
     guint modifiers = 0;
 
@@ -732,17 +688,8 @@ _ic_process_key_event  (BusInputContext       *context,
         }
     }
 
-    if (G_LIKELY (context->has_focus)) {
-        gboolean retval = bus_input_context_filter_keyboard_shortcuts (context, keyval, keycode, modifiers);
-        /* If it is keyboard shortcut, reply TRUE to client */
-        if (G_UNLIKELY (retval)) {
-            g_dbus_method_invocation_return_value (invocation, g_variant_new ("(b)", TRUE));
-            return;
-        }
-    }
-
     /* ignore key events, if it is a fake input context */
-    if (context->has_focus && context->enabled && context->engine && context->fake == FALSE) {
+    if (context->has_focus && context->engine && context->fake == FALSE) {
         bus_engine_proxy_process_key_event (context->engine,
                                             keyval,
                                             keycode,
@@ -770,7 +717,7 @@ _ic_set_cursor_location (BusInputContext       *context,
     g_variant_get (parameters, "(iiii)",
                    &context->x, &context->y, &context->w, &context->h);
 
-    if (context->has_focus && context->enabled && context->engine) {
+    if (context->has_focus && context->engine) {
         bus_engine_proxy_set_cursor_location (context->engine,
                         context->x, context->y, context->w, context->h);
     }
@@ -792,7 +739,7 @@ _ic_process_hand_writing_event (BusInputContext       *context,
                                 GDBusMethodInvocation *invocation)
 {
     /* do nothing if it is a fake input context */
-    if (context->has_focus && context->enabled &&
+    if (context->has_focus &&
         context->engine && context->fake == FALSE) {
         bus_engine_proxy_process_hand_writing_event (context->engine, parameters);
     }
@@ -808,7 +755,7 @@ _ic_cancel_hand_writing (BusInputContext       *context,
     g_variant_get (parameters, "(u)", &n_strokes);
 
     /* do nothing if it is a fake input context */
-    if (context->has_focus && context->enabled &&
+    if (context->has_focus &&
         context->engine && context->fake == FALSE) {
         bus_engine_proxy_cancel_hand_writing (context->engine, n_strokes);
     }
@@ -865,7 +812,7 @@ _ic_reset (BusInputContext       *context,
            GVariant              *parameters,
            GDBusMethodInvocation *invocation)
 {
-    if (context->enabled && context->engine) {
+    if (context->engine) {
         bus_engine_proxy_reset (context->engine);
     }
     g_dbus_method_invocation_return_value (invocation, NULL);
@@ -903,7 +850,7 @@ _ic_property_activate (BusInputContext       *context,
     guint prop_state = 0;
     g_variant_get (parameters, "(&su)", &prop_name, &prop_state);
 
-    if (context->enabled && context->engine) {
+    if (context->engine) {
         bus_engine_proxy_property_activate (context->engine, prop_name, prop_state);
     }
 
@@ -921,47 +868,6 @@ _ic_property_activate (BusInputContext       *context,
     g_dbus_method_invocation_return_value (invocation, NULL);
 }
 
-/**
- * _ic_enable:
- *
- * Implement the "Enable" method call of the org.freedesktop.IBus.InputContext interface.
- */
-static void
-_ic_enable (BusInputContext       *context,
-            GVariant              *parameters,
-            GDBusMethodInvocation *invocation)
-{
-    bus_input_context_enable (context);
-    g_dbus_method_invocation_return_value (invocation, NULL);
-}
-
-/**
- * _ic_disable:
- *
- * Implement the "Disable" method call of the org.freedesktop.IBus.InputContext interface.
- */
-static void
-_ic_disable (BusInputContext       *context,
-             GVariant              *parameters,
-             GDBusMethodInvocation *invocation)
-{
-    bus_input_context_disable (context);
-    g_dbus_method_invocation_return_value (invocation, NULL);
-}
-
-/**
- * _ic_is_enabled:
- *
- * Implement the "IsEnabled" method call of the org.freedesktop.IBus.InputContext interface.
- */
-static void
-_ic_is_enabled (BusInputContext       *context,
-                GVariant              *parameters,
-                GDBusMethodInvocation *invocation)
-{
-    g_dbus_method_invocation_return_value (invocation, g_variant_new ("(b)", context->enabled));
-}
-
 static void
 _ic_set_engine_done (BusInputContext       *context,
                      GAsyncResult          *res,
@@ -1033,18 +939,13 @@ _ic_get_engine (BusInputContext       *context,
                 GVariant              *parameters,
                 GDBusMethodInvocation *invocation)
 {
-    if (context->engine) {
-        IBusEngineDesc *desc = bus_engine_proxy_get_desc (context->engine);
-        g_dbus_method_invocation_return_value (invocation,
-                        g_variant_new ("(v)", ibus_serializable_serialize ((IBusSerializable *)desc)));
-    }
-    else {
-        g_dbus_method_invocation_return_error (
-                invocation,
-                IBUS_ERROR,
-                IBUS_ERROR_NO_ENGINE,
-                "Input context does not have engine.");
-    }
+    IBusEngineDesc *desc = context->engine ?
+            bus_engine_proxy_get_desc (context->engine) :
+            BUS_INPUT_CONTEXT_GET_CLASS (context)->default_engine_desc;
+
+
+    g_dbus_method_invocation_return_value (invocation,
+            g_variant_new ("(v)", ibus_serializable_serialize ((IBusSerializable *)desc)));
 }
 
 /**
@@ -1071,7 +972,7 @@ _ic_set_surrounding_text (BusInputContext       *context,
     g_variant_unref (variant);
 
     if ((context->capabilities & IBUS_CAP_SURROUNDING_TEXT) &&
-         context->has_focus && context->enabled && context->engine) {
+         context->has_focus && context->engine) {
         bus_engine_proxy_set_surrounding_text (context->engine,
                                                text,
                                                cursor_pos,
@@ -1094,7 +995,7 @@ bus_input_context_service_method_call (IBusService            *service,
                                        GVariant               *parameters,
                                        GDBusMethodInvocation  *invocation)
 {
-    if (g_strcmp0 (interface_name, "org.freedesktop.IBus.InputContext") != 0) {
+    if (g_strcmp0 (interface_name, IBUS_INTERFACE_INPUT_CONTEXT) != 0) {
         IBUS_SERVICE_CLASS (bus_input_context_parent_class)->service_method_call (
                         service,
                         connection,
@@ -1121,9 +1022,6 @@ bus_input_context_service_method_call (IBusService            *service,
         { "Reset",             _ic_reset },
         { "SetCapabilities",   _ic_set_capabilities },
         { "PropertyActivate",  _ic_property_activate },
-        { "Enable",            _ic_enable },
-        { "Disable",           _ic_disable },
-        { "IsEnabled",         _ic_is_enabled },
         { "SetEngine",         _ic_set_engine },
         { "GetEngine",         _ic_get_engine },
         { "SetSurroundingText", _ic_set_surrounding_text},
@@ -1160,28 +1058,10 @@ bus_input_context_focus_in (BusInputContext *context)
 
     /* To make sure that we won't use an old value left before we losing focus
      * last time. */
-    context->prev_keyval = IBUS_VoidSymbol;
+    context->prev_keyval = IBUS_KEY_VoidSymbol;
     context->prev_modifiers = 0;
 
-    if (context->engine == NULL && context->enabled) {
-        /* request an engine, e.g. a global engine if the feature is enabled. */
-        IBusEngineDesc *desc = NULL;
-        g_signal_emit (context,
-                       context_signals[REQUEST_ENGINE], 0,
-                       NULL,
-                       &desc);
-
-        if (desc != NULL) {
-            bus_input_context_set_engine_by_desc (context,
-                            desc,
-                            g_gdbus_timeout, /* timeout in msec. */
-                            NULL, /* we do not cancel the call. */
-                            NULL, /* use the default callback function. */
-                            NULL);
-        }
-    }
-
-    if (context->engine && context->enabled) {
+    if (context->engine) {
         bus_engine_proxy_focus_in (context->engine);
         bus_engine_proxy_enable (context->engine);
         bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
@@ -1190,7 +1070,7 @@ bus_input_context_focus_in (BusInputContext *context)
 
     if (context->capabilities & IBUS_CAP_FOCUS) {
         g_signal_emit (context, context_signals[FOCUS_IN], 0);
-        if (context->engine && context->enabled) {
+        if (context->engine) {
             /* if necessary, emit glib signals to the context object to update panel status. see the comment for PREEDIT_CONDITION
              * for details. */
             if (context->preedit_visible && !PREEDIT_CONDITION) {
@@ -1252,7 +1132,7 @@ bus_input_context_focus_out (BusInputContext *context)
     bus_input_context_update_lookup_table (context, lookup_table_empty, FALSE);
     bus_input_context_register_properties (context, props_empty);
 
-    if (context->engine && context->enabled) {
+    if (context->engine) {
         bus_engine_proxy_focus_out (context->engine);
     }
 
@@ -1269,7 +1149,7 @@ bus_input_context_focus_out (BusInputContext *context)
     {                                                                       \
         g_assert (BUS_IS_INPUT_CONTEXT (context));                          \
                                                                             \
-        if (context->has_focus && context->enabled && context->engine) {    \
+        if (context->has_focus && context->engine) {    \
             bus_engine_proxy_##name (context->engine);                      \
         }                                                                   \
     }
@@ -1289,7 +1169,7 @@ bus_input_context_candidate_clicked (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (context->enabled && context->engine) {
+    if (context->engine) {
         bus_engine_proxy_candidate_clicked (context->engine,
                                             index,
                                             button,
@@ -1304,7 +1184,7 @@ bus_input_context_property_activate (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (context->enabled && context->engine) {
+    if (context->engine) {
         bus_engine_proxy_property_activate (context->engine, prop_name, prop_state);
     }
 }
@@ -1315,9 +1195,6 @@ bus_input_context_commit_text (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (!context->enabled)
-        return;
-
     if (text == text_empty || text == NULL)
         return;
 
@@ -1829,9 +1706,6 @@ _engine_forward_key_event_cb (BusEngineProxy    *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_emit_signal (context,
                                    "ForwardKeyEvent",
                                    g_variant_new ("(uuu)", keyval, keycode, state),
@@ -1854,9 +1728,6 @@ _engine_delete_surrounding_text_cb (BusEngineProxy    *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_emit_signal (context,
                                    "DeleteSurroundingText",
                                    g_variant_new ("(iu)", offset_from_cursor, nchars),
@@ -1877,9 +1748,6 @@ _engine_require_surrounding_text_cb (BusEngineProxy    *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_emit_signal (context,
                                    "RequireSurroundingText",
                                    NULL,
@@ -1905,9 +1773,6 @@ _engine_update_preedit_text_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_update_preedit_text (context, text, cursor_pos, visible, mode);
 }
 
@@ -1928,9 +1793,6 @@ _engine_update_auxiliary_text_cb (BusEngineProxy   *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_update_auxiliary_text (context, text, visible);
 }
 
@@ -1951,9 +1813,6 @@ _engine_update_lookup_table_cb (BusEngineProxy   *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_update_lookup_table (context, table, visible);
 }
 
@@ -1973,9 +1832,6 @@ _engine_register_properties_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_register_properties (context, props);
 }
 
@@ -1995,9 +1851,6 @@ _engine_update_property_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    if (!context->enabled)
-        return;
-
     bus_input_context_update_property (context, prop);
 }
 
@@ -2011,9 +1864,6 @@ _engine_update_property_cb (BusEngineProxy  *engine,
                                                                 \
         g_assert (context->engine == engine);                   \
                                                                 \
-        if (!context->enabled)                                  \
-            return;                                             \
-                                                                \
         bus_input_context_##name (context);                     \
     }
 
@@ -2077,8 +1927,6 @@ bus_input_context_enable (BusInputContext *context)
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
     if (!context->has_focus) {
-        context->enabled = TRUE;
-        /* FIXME Do we need to emit "enabled" signal? */
         return;
     }
 
@@ -2101,20 +1949,10 @@ bus_input_context_enable (BusInputContext *context)
     if (context->engine == NULL)
         return;
 
-    context->enabled = TRUE;
-
     bus_engine_proxy_focus_in (context->engine);
     bus_engine_proxy_enable (context->engine);
     bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
     bus_engine_proxy_set_cursor_location (context->engine, context->x, context->y, context->w, context->h);
-
-    bus_input_context_emit_signal (context,
-                                   "Enabled",
-                                   NULL,
-                                   NULL);
-    g_signal_emit (context,
-                   context_signals[ENABLED],
-                   0);
 }
 
 void
@@ -2131,24 +1969,6 @@ bus_input_context_disable (BusInputContext *context)
         bus_engine_proxy_focus_out (context->engine);
         bus_engine_proxy_disable (context->engine);
     }
-
-    bus_input_context_emit_signal (context,
-                                   "Disabled",
-                                   NULL,
-                                   NULL);
-    g_signal_emit (context,
-                   context_signals[DISABLED],
-                   0);
-
-    context->enabled = FALSE;
-}
-
-gboolean
-bus_input_context_is_enabled (BusInputContext *context)
-{
-    g_assert (BUS_IS_INPUT_CONTEXT (context));
-
-    return context->enabled;
 }
 
 /* A list of signals (and their handler functions) that could be emit by the engine proxy object. */
@@ -2176,7 +1996,6 @@ const static struct {
     { "register-properties",      G_CALLBACK (_engine_register_properties_cb) },
     { "update-property",          G_CALLBACK (_engine_update_property_cb) },
     { "destroy",                  G_CALLBACK (_engine_destroy_cb) },
-    { NULL, 0 }
 };
 
 static void
@@ -2192,9 +2011,14 @@ bus_input_context_unset_engine (BusInputContext *context)
     if (context->engine) {
         gint i;
         /* uninstall signal handlers for the engine. */
-        for (i = 0; engine_signals[i].name != NULL; i++) {
-            g_signal_handlers_disconnect_by_func (context->engine, engine_signals[i].callback, context);
+        for (i = 0; i < G_N_ELEMENTS(engine_signals); i++) {
+            g_signal_handlers_disconnect_by_func (context->engine,
+                    engine_signals[i].callback, context);
         }
+        /* focus out engine so that the next call of
+           bus_engine_proxy_focus_in() will take effect and trigger
+           RegisterProperties. */
+        bus_engine_proxy_focus_out (context->engine);
         g_object_unref (context->engine);
         context->engine = NULL;
     }
@@ -2222,13 +2046,13 @@ bus_input_context_set_engine (BusInputContext *context,
         g_object_ref (context->engine);
 
         /* handle signals from the engine. */
-        for (i = 0; engine_signals[i].name != NULL; i++) {
+        for (i = 0; i < G_N_ELEMENTS(engine_signals); i++) {
             g_signal_connect (context->engine,
                               engine_signals[i].name,
                               engine_signals[i].callback,
                               context);
         }
-        if (context->has_focus && context->enabled) {
+        if (context->has_focus) {
             bus_engine_proxy_focus_in (context->engine);
             bus_engine_proxy_enable (context->engine);
             bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
@@ -2292,12 +2116,8 @@ new_engine_cb (GObject             *obj,
                                              "Opertation was cancelled");
         }
         else {
-            /* Let BusEngineProxy call a Disable signal.
-             * The Disable signal is needed when the engines are switched
-             * but not needed when an engine is enabled initially. */
-            if (data->context->engine) {
-                bus_input_context_disable (data->context);
-            }
+            /* Let BusEngineProxy call a Disable signal. */
+            bus_input_context_disable (data->context);
             bus_input_context_set_engine (data->context, engine);
             g_object_unref (engine);
             bus_input_context_enable (data->context);
@@ -2465,44 +2285,6 @@ bus_input_context_get_engine_desc (BusInputContext *context)
     return NULL;
 }
 
-static gboolean
-bus_input_context_filter_keyboard_shortcuts (BusInputContext    *context,
-                                             guint               keyval,
-                                             guint               keycode,
-                                             guint               modifiers)
-{
-    g_assert (BUS_IS_INPUT_CONTEXT (context));
-
-    gboolean retval = FALSE;
-
-    if (context->filter_release){
-        if (modifiers & IBUS_RELEASE_MASK) {
-            /* filter release key event */
-            return TRUE;
-        }
-        else {
-            /* stop filter release key event */
-            context->filter_release = FALSE;
-        }
-    }
-
-    retval = bus_ibus_impl_filter_keyboard_shortcuts (BUS_DEFAULT_IBUS,
-                                                      context,
-                                                      keyval,
-                                                      modifiers,
-                                                      context->prev_keyval,
-                                                      context->prev_modifiers);
-    context->prev_keyval = keyval;
-    context->prev_modifiers = modifiers;
-
-    if (retval == TRUE) {
-        /* begin filter release key event */
-        context->filter_release = TRUE;
-    }
-
-    return retval;
-}
-
 guint
 bus_input_context_get_capabilities (BusInputContext *context)
 {
