@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
 #ifndef __IBUS_H_
 #define __IBUS_H_
 
@@ -35,6 +36,7 @@
 #include <ibusservice.h>
 #include <ibusfactory.h>
 #include <ibusengine.h>
+#include <ibusenginesimple.h>
 #include <ibuserror.h>
 #include <ibusproperty.h>
 #include <ibusproplist.h>
@@ -53,6 +55,11 @@
 #include <ibusconfigservice.h>
 #include <ibuspanelservice.h>
 #include <ibusutil.h>
+#include <ibusregistry.h>
+
+#ifndef IBUS_DISABLE_DEPRECATED
+#include <ibuskeysyms-compat.h>
+#endif
 
 #undef __IBUS_H_INSIDE__
 
