@@ -86,10 +86,15 @@ typedef struct _IBusEngineDescClass IBusEngineDescClass;
  * author: Author of the input method engine.
  * icon: Icon file of this engine.
  * layout: Keyboard layout
+ * layout_variant: Keyboard variant
+ * layout_option: Keyboard option
  * rank: Preference rank among engines, the highest ranked IME will put in
  * the front.
  * hotkeys: One or more hotkeys for switching to this engine, separated by
  *  semi-colon.
+ * setup: Exec lists of the engine setup command.
+ * version: Version number of the input method engine.
+ * textdomain: Domain name for dgettext()
  */
 struct _IBusEngineDesc {
     IBusSerializable parent;
@@ -230,6 +235,26 @@ const gchar     *ibus_engine_desc_get_icon      (IBusEngineDesc *info);
  */
 const gchar     *ibus_engine_desc_get_layout    (IBusEngineDesc *info);
 
+/**
+ * ibus_engine_desc_get_layout_variant:
+ * @info: An IBusEngineDesc
+ * @returns: keyboard variant property in IBusEngineDesc
+ *
+ * Return the keyboard variant property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_layout_variant
+                                                (IBusEngineDesc *info);
+
+/**
+ * ibus_engine_desc_get_layout_option:
+ * @info: An IBusEngineDesc
+ * @returns: keyboard option property in IBusEngineDesc
+ *
+ * Return the keyboard option property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_layout_option
+                                                (IBusEngineDesc *info);
+
 /**
  * ibus_engine_desc_get_rank:
  * @info: An IBusEngineDesc
@@ -248,6 +273,43 @@ guint            ibus_engine_desc_get_rank      (IBusEngineDesc *info);
  */
 const gchar     *ibus_engine_desc_get_hotkeys   (IBusEngineDesc *info);
 
+/**
+ * ibus_engine_desc_get_symbol:
+ * @info: An IBusEngineDesc
+ * @returns: symbol property in IBusEngineDesc
+ *
+ * Return the symbol property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_symbol    (IBusEngineDesc *info);
+
+/**
+ * ibus_engine_desc_get_setup:
+ * @info: An IBusEngineDesc
+ * @returns: setup property in IBusEngineDesc
+ *
+ * Return the setup property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_setup     (IBusEngineDesc *info);
+
+/**
+ * ibus_engine_desc_get_version:
+ * @info: An IBusEngineDesc
+ * @returns: version in IBusEngineDesc
+ *
+ * Return the version property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_version   (IBusEngineDesc *info);
+
+/**
+ * ibus_engine_desc_get_textdomain:
+ * @info: An IBusEngineDesc
+ * @returns: textdomain in IBusEngineDesc
+ *
+ * Return the textdomain property in IBusEngineDesc. It should not be freed.
+ */
+const gchar     *ibus_engine_desc_get_textdomain
+                                                (IBusEngineDesc *info);
+
 /**
  * ibus_engine_desc_output:
  * @info: An IBusEngineDesc
