@@ -275,7 +275,9 @@ ibus_free_strv (gchar **strv)
 void
 ibus_init (void)
 {
+#if !GLIB_CHECK_VERSION(2,35,0)
     g_type_init ();
+#endif
     IBUS_ERROR;
     IBUS_TYPE_TEXT;
     IBUS_TYPE_ATTRIBUTE;
@@ -283,6 +285,8 @@ ibus_init (void)
     IBUS_TYPE_LOOKUP_TABLE;
     IBUS_TYPE_COMPONENT;
     IBUS_TYPE_ENGINE_DESC;
+    IBUS_TYPE_OBSERVED_PATH;
+    IBUS_TYPE_REGISTRY;
 }
 
 static GMainLoop *main_loop = NULL;
